/****************************************************************************
** Copyright (C) 2020 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** Permission is hereby granted, free of charge, to any person obtaining a copy
** of this software and associated documentation files (the "Software"), to deal
** in the Software without restriction, including without limitation the rights
** to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
** copies of the Software, and to permit persons to whom the Software is
** furnished to do so, subject to the following conditions:
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
**  USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

/*!
 * @file ph2.c
 * @brief pH 2 Click Driver.
 */

#include "ph2.h"

void ph2_cfg_setup ( ph2_cfg_t *cfg )
{
    cfg->an  = HAL_PIN_NC;
    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    cfg->st1 = HAL_PIN_NC;
    cfg->st2 = HAL_PIN_NC;

    cfg->resolution  = ANALOG_IN_RESOLUTION_DEFAULT; /*< Should leave this by default for portability purposes. 
                                                         Different MCU's have different resolutions. 
                                                         Change only if necessary.*/
    cfg->i2c_speed   = I2C_MASTER_SPEED_STANDARD;
    cfg->i2c_address = PH2_SET_DEV_ADDR;
    cfg->vref        = PH2_VREF_3V3;
    cfg->drv_sel     = PH2_DRV_SEL_I2C;
}

void ph2_drv_interface_selection ( ph2_cfg_t *cfg, ph2_drv_t drv_sel ) 
{
    cfg->drv_sel = drv_sel;
}

err_t ph2_init ( ph2_t *ctx, ph2_cfg_t *cfg ) 
{
    ctx->drv_sel = cfg->drv_sel;
    ctx->vref = cfg->vref;

    if ( PH2_DRV_SEL_I2C == ctx->drv_sel ) 
    {
        i2c_master_config_t i2c_cfg;

        i2c_master_configure_default( &i2c_cfg );

        i2c_cfg.scl = cfg->scl;
        i2c_cfg.sda = cfg->sda;

        ctx->slave_address = cfg->i2c_address;

        if ( I2C_MASTER_ERROR == i2c_master_open( &ctx->i2c, &i2c_cfg ) ) 
        {
            return I2C_MASTER_ERROR;
        }

        if ( I2C_MASTER_ERROR == i2c_master_set_slave_address( &ctx->i2c, ctx->slave_address ) ) 
        {
            return I2C_MASTER_ERROR;
        }

        if ( I2C_MASTER_ERROR == i2c_master_set_speed( &ctx->i2c, cfg->i2c_speed ) ) 
        {
            return I2C_MASTER_ERROR;
        }
    }
    else
    {
        analog_in_config_t adc_cfg;

        analog_in_configure_default( &adc_cfg );

        adc_cfg.input_pin = cfg->an;

        if ( ADC_ERROR == analog_in_open( &ctx->adc, &adc_cfg ) ) 
        {
            return ADC_ERROR;
        }

        if ( ADC_ERROR == analog_in_set_vref_value( &ctx->adc, cfg->vref ) ) 
        {
            return ADC_ERROR;
        }

        if ( ADC_ERROR == analog_in_set_resolution( &ctx->adc, cfg->resolution ) ) 
        {
            return ADC_ERROR;
        }
    }

    digital_out_init( &ctx->st1, cfg->st1 );
    digital_out_init( &ctx->st2, cfg->st2 );

    return PH2_OK;
}

err_t ph2_read_raw_adc ( ph2_t *ctx, uint16_t *raw_adc ) 
{
    if ( PH2_DRV_SEL_I2C == ctx->drv_sel ) 
    {
        uint8_t rx_buf[ 2 ];
        err_t err_flag = i2c_master_read( &ctx->i2c, rx_buf, 2 );
        *raw_adc = ( ( ( uint16_t ) rx_buf[ 0 ] << 8 ) | ( rx_buf[ 1 ] & 0xFF ) ) & PH2_ADC_RESOLUTION;
        return err_flag;
    }
    else
    {
        return analog_in_read( &ctx->adc, raw_adc );
    }
}

err_t ph2_read_voltage ( ph2_t *ctx, float *voltage ) 
{
    if ( PH2_DRV_SEL_I2C == ctx->drv_sel ) 
    {
        uint16_t raw_adc;
        err_t err_flag = ph2_read_raw_adc( ctx, &raw_adc );
        *voltage = ( float ) raw_adc * ( ctx->vref / ( float ) PH2_ADC_RESOLUTION );
        return err_flag;
    }
    else
    {
        return analog_in_read_voltage( &ctx->adc, voltage );
    }
}

err_t ph2_set_vref ( ph2_t *ctx, float vref ) 
{
    ctx->vref = vref;
    if ( PH2_DRV_SEL_I2C == ctx->drv_sel ) 
    {
        return PH2_OK;
    }
    else
    {
        return analog_in_set_vref_value( &ctx->adc, vref );
    }
}

err_t ph2_calibrate ( ph2_t *ctx, float pH_value )
{
    err_t error_flag = PH2_OK;
    float rx_data;
    
    error_flag = ph2_read_voltage( ctx, &rx_data );
    ctx->pH_vol = rx_data;
    ctx->pH_val = pH_value;

    return error_flag;
}

err_t ph2_calculate_ph ( ph2_t *ctx, float *pH_value )
{
    err_t error_flag = PH2_OK;
    float rx_data;
    
    error_flag = ph2_read_voltage( ctx, &rx_data );
    *pH_value = ctx->pH_val + ( (ctx->pH_vol - rx_data) / PH2_PH_CALCULATION_COEF);
    
    return error_flag;
}

void ph2_calibrate_offset ( ph2_t *ctx )
{
    float rx_data= 0;
    while ( ( (uint8_t) ( rx_data * 100)  ) != ( (uint8_t) ( ctx->vref * 50 ) ) )
    {
        ph2_read_voltage( ctx, &rx_data );
        if ( ( (uint8_t) ( rx_data * 100 ) ) > ( (uint8_t) ( ctx->vref * 50 ) ) )
        {
            ph2_enable_st2( ctx );
            Delay_100ms( );
            ph2_disable_st2( ctx );
            Delay_100ms( );
        }
        if ( ( (uint8_t) ( rx_data * 100 ) ) < ( (uint8_t) ( ctx->vref * 50 ) ) )
        {
            ph2_enable_st1( ctx );
            Delay_100ms( );
            ph2_disable_st1( ctx );
            Delay_100ms( );
        }
    }
    ph2_enable_st1( ctx );
    ph2_enable_st2( ctx );
    Delay_100ms( );
    ph2_disable_st1( ctx );
    ph2_disable_st2( ctx );
}

void ph2_enable_st1 ( ph2_t *ctx )
{
    digital_out_high( &ctx->st1 );
}

void ph2_disable_st1 ( ph2_t *ctx )
{
    digital_out_low( &ctx->st1 );
}

void ph2_enable_st2 ( ph2_t *ctx )
{
    digital_out_high( &ctx->st2 );
}

void ph2_disable_st2 ( ph2_t *ctx )
{
    digital_out_low( &ctx->st2 );
}

// ------------------------------------------------------------------------- END
